
#define _cs  12
#define _dc  A0
#define _rst 10

#include <Arduino.h>
#include <SPI.h>
#include "gfx.h"

long framecount=0;
char frameBuffer[512]; // single tile row buffer

//TODO: Find a way to handle the write commands without toggling CS and DC every time
void writeCmd(uint8_t reg){//Writes a command byte to the driver
  digitalWrite(_dc, LOW);
  digitalWrite(_cs, LOW);
  SPI.transfer(reg);
  digitalWrite(_cs, HIGH);
}

void writeData(uint8_t data){//Writes 1 byte to the display's memory
  digitalWrite(_dc, HIGH);
  digitalWrite(_cs, LOW);
  SPI.transfer(data);
  digitalWrite(_cs, HIGH);
}

void setContrast(uint8_t contrast){
  writeCmd(0x81);  //set contrast control
  writeCmd(contrast);  //Contrast byte
}

void setWriteZone(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2) { //defines a rectangular area of memory which the driver will itterate through. This function takes memory locations, meaning a 64x128 space
  writeCmd(0x15); //Set Column Address
  writeCmd(x1); //Beginning. Note that you must divide the column by 2, since 1 byte in memory is 2 pixels
  writeCmd(x2); //End
  
  writeCmd(0x75); //Set Row Address
  writeCmd(y1); //Beginning
  writeCmd(y2); //End
}


void initRegs(){ //Sends all the boilerplate startup and config commands to the driver
  writeCmd(0xae);//--turn off oled panel

  writeCmd(0x15);  //set column addresses
  writeCmd(0x00);  //start column  0
  writeCmd(0x7f);  //end column  127

  writeCmd(0x75);  //set row addresses
  writeCmd(0x00);  //start row  0
  writeCmd(0x7f);  //end row  127

  writeCmd(0x81);  //set contrast control
  writeCmd(0x80);  //50% (128/255)

  writeCmd(0xa0);   //gment remap
  writeCmd(0x51);  //51 (To my understanding, this is orientation

  writeCmd(0xa1);  //start line
  writeCmd(0x00);

  writeCmd(0xa2);  //display offset
  writeCmd(0x00);

  writeCmd(0xa4);  //rmal display
  writeCmd(0xa8);  //set multiplex ratio
  writeCmd(0x7f);

  writeCmd(0xb1);  //set phase leghth
  writeCmd(0xf1);

  writeCmd(0xb3);  //set dclk
  writeCmd(0x00);  //80Hz:0xc1 90Hz:0xe1  100Hz:0x00  110Hz:0x30 120Hz:0x50  130Hz:0x70   01

  writeCmd(0xab);  //Enable vReg
  writeCmd(0x01);  

  writeCmd(0xb6);  //set phase leghth
  writeCmd(0x0f);

  writeCmd(0xbe); //Set vcomh voltage
  writeCmd(0x0f);

  writeCmd(0xbc); //set pre-charge voltage
  writeCmd(0x08);

  writeCmd(0xd5); //second precharge period
  writeCmd(0x62);

  writeCmd(0xfd); //Unlock commands
  writeCmd(0x12);

  writeCmd(0xAF);
  delay(300);
}

void setup() {
  pinMode(_cs, OUTPUT);
  pinMode(_dc, OUTPUT);
  pinMode(_rst, OUTPUT);
  
  SPI.setDataMode(SPI_MODE0);
  SPI.setBitOrder(MSBFIRST);
  SPI.setClockDivider(SPI_CLOCK_DIV2);
  SPI.begin();
  
  digitalWrite(_rst, HIGH); //Reset display
  delay(200);
  digitalWrite(_rst, LOW);
  delay(200);
  digitalWrite(_rst, HIGH);
  delay(200);

  initRegs();
}

void paintTile(int x1, int tile){
  int j=2;
  for(int y=0; y<8; y++){
    for(int x=0; x<4; x++){
      byte pix2 = tileset[tile][j++];
      byte hiWord = palette[pix2 >> 4];
      byte loWord = palette[pix2 & 15];
      frameBuffer[x1+x+64*y] = (hiWord<<4)|loWord;
    }
  }
}

void drawFullMap(){
  for(int y=0; y<16; y++){
    for(int x=0; x<16; x++){
      paintTile(x*4,myMap[x+16*y]);
    }
    setWriteZone(0,y*8,127,127); //Full display
    digitalWrite(_dc, HIGH);
    digitalWrite(_cs, LOW);
    for(int i = 0; i < 512; i++){ SPI.transfer(frameBuffer[i]); }
    digitalWrite(_cs, HIGH);
  }
}

void clearScreen(){
  setWriteZone(0,0,127,127); //Full display
  digitalWrite(_dc, HIGH);
  digitalWrite(_cs, LOW);
  for(int i = 0; i < 8192; i++){ SPI.transfer(0);}
  digitalWrite(_cs, HIGH);
}

void writeFullBuffer(){ //Outputs the full framebuffer to the display
  setWriteZone(0,0,127,127); //Full display
  digitalWrite(_dc, HIGH);
  digitalWrite(_cs, LOW);
  for(int i = 0; i < 512; i++){ SPI.transfer(frameBuffer[i]); }
  digitalWrite(_cs, HIGH);


}

void loop() {
  drawFullMap();
  clearScreen();
}
